# -*- coding: iso-8859-15 -*-

"""Module with the configuration.

This exports:
  - platform -- platform dependent values
  - default -- default values
  - options -- current values

Copyright (C) 2005, 2006  Xos Otero <xoseotero@users.sourceforge.net>

"""

import sys
import os.path
import ConfigParser

__all__ = ["platform", "default", "options"]


class Config(ConfigParser.SafeConfigParser):
    def __init__(self, syscfg, platform=None, usercfg=None):
        """

        Arguments:
        syscfg -- system configuration file

        Keyword arguments:
        platform -- platform specific configuration
        usercfg -- user configuration file

        """
        ConfigParser.SafeConfigParser.__init__(self, self.__platform_defaults(platform))
        ConfigParser.SafeConfigParser.readfp(self, file(syscfg))
        if usercfg:
            ConfigParser.SafeConfigParser.read(self, usercfg)

    def __platform_defaults(self, platform):
        """

        """
        defaults = {}
        if platform:
            for option in platform.options("paths"):
                defaults[option] = platform.get("paths", option)
        return defaults


def platform_configuration():
    """Return the path of platform.cfg

    platform.cfg must be in the package pythonsudoku

    """
    for path in sys.path:
        platform_cfg = os.path.join(path, "pythonsudoku", "platform.cfg")
        if os.path.isfile(platform_cfg):
            return platform_cfg


platform = Config(platform_configuration())
default = Config(platform.get("paths", "syscfg"), platform)
options = Config(platform.get("paths", "syscfg"), platform,
                 platform.get("paths", "usercfg"))
