# -*- coding: iso-8859-15 -*-

"""Module for Graphical User Interface.

This exports the class:
  - Gui -- the GUI

Copyright (C) 2005, 2006  Xos Otero <xoseotero@users.sourceforge.net>

"""

__all__ = ["Gui"]

import pygtk
pygtk.require('2.0')
import gtk
import pango

from check_modules import *
from board import Board, Value
from sudoku import Sudoku
from history import History
if has_reportlab:
    from pdf import PDF
    from printer import Printer
if has_PIL:
    from image import Image
from info import Info
from config import options

class Menu(object):
    __slots__ = ["gui", "menu_bar"]

    def __init__(self, gui):
        self.gui = gui

        self.menu_bar = gtk.MenuBar()
        self.menu_bar.show()

        self.file_menu()
        self.edit_menu()
        self.sudoku_menu()
        self.help_menu()

    def file_menu(self):
        file_menu = gtk.Menu()
        open_item = gtk.MenuItem(_("_Open"))
        open_item.connect("activate", self.gui.callback, "open")
        open_item.show()
        file_menu.append(open_item)
        save_sdk_item = gtk.MenuItem(_("_Save sudoku"))
        save_sdk_item.connect("activate", self.gui.callback, "save")
        save_sdk_item.show()
        file_menu.append(save_sdk_item)
        if has_reportlab:
            save_pdf_item = gtk.MenuItem(_("Save as P_DF"))
            save_pdf_item.connect("activate", self.gui.callback, "save_pdf")
            save_pdf_item.show()
            file_menu.append(save_pdf_item)
        if has_PIL:
            save_image_item = gtk.MenuItem(_("Save as _Image"))
            save_image_item.connect("activate", self.gui.callback,
                                    "save_image")
            save_image_item.show()
            file_menu.append(save_image_item)
        if has_reportlab:
            print_item = gtk.MenuItem(_("_Print"))
            print_item.connect("activate", self.gui.callback, "print")
            print_item.show()
            file_menu.append(print_item)
        quit_item = gtk.MenuItem(_("_Quit"))
        quit_item.show()
        quit_item.connect("activate", self.gui.callback, "quit")
        file_menu.append(quit_item)

        file_item = gtk.MenuItem(_("_File"))
        file_item.set_submenu(file_menu)
        file_item.show()

        self.menu_bar.append(file_item)

    def edit_menu(self):
        edit_menu = gtk.Menu()
        undo_item = gtk.MenuItem(_("_Undo"))
        undo_item.connect("activate", self.gui.callback, "undo")
        undo_item.show()
        edit_menu.append(undo_item)
        redo_item = gtk.MenuItem(_("_Redo"))
        redo_item.connect("activate", self.gui.callback, "redo")
        redo_item.show()
        edit_menu.append(redo_item)

        edit_item = gtk.MenuItem(_("_Edit"))
        edit_item.set_submenu(edit_menu)
        edit_item.show()

        self.menu_bar.append(edit_item)

    def sudoku_menu(self):
        sudoku_menu = gtk.Menu()
        create_item = gtk.MenuItem(_("_Create"))
        create_item.connect("activate", self.gui.callback, "create")
        create_item.show()
        sudoku_menu.append(create_item)
        check_item = gtk.MenuItem(_("C_heck"))
        check_item.connect("activate", self.gui.callback, "check")
        check_item.show()
        sudoku_menu.append(check_item)
        solve_item = gtk.MenuItem(_("_Solve"))
        solve_item.connect("activate", self.gui.callback, "solve")
        solve_item.show()
        sudoku_menu.append(solve_item)
        give_one_item = gtk.MenuItem(_("_Give one number"))
        give_one_item.connect("activate", self.gui.callback, "give_one")
        give_one_item.show()
        sudoku_menu.append(give_one_item)

        sudoku_item = gtk.MenuItem(_("_Sudoku"))
        sudoku_item.set_submenu(sudoku_menu)
        sudoku_item.show()

        self.menu_bar.append(sudoku_item)

    def help_menu(self):
        help_menu = gtk.Menu()
        about_item = gtk.MenuItem(_("_About"))
        about_item.connect("activate", self.gui.callback, "about")
        about_item.show()
        help_menu.append(about_item)
        whatis_item = gtk.MenuItem(_("_What is"))
        whatis_item.connect("activate", self.gui.callback, "whatis")
        whatis_item.show()
        help_menu.append(whatis_item)

        help_item = gtk.MenuItem(_("_Help"))
        help_item.set_submenu(help_menu)
        help_item.show()

        self.menu_bar.append(help_item)

    def widget(self):
        return self.menu_bar

class OpenWindow(object):
    __slots__ = ["gui", "filesel"]

    def __init__(self, gui):
        self.gui = gui

        self.filesel = gtk.FileSelection(_("Open file"))
        self.filesel.ok_button.connect("clicked", self.callback)
        self.filesel.cancel_button.connect("clicked",
                                           lambda w: self.filesel.destroy())

        self.filesel.show()

    def callback(self, widget, data=None):
        self.gui.load(self.filesel.get_filename())
        self.filesel.destroy()

class SaveWindow(object):
    __slots__ = ["gui", "filesel"]

    def __init__(self, gui):
        self.gui = gui

        self.filesel = gtk.FileSelection(_("Save file"))
        self.filesel.ok_button.connect("clicked", self.callback)
        self.filesel.cancel_button.connect("clicked",
                                           lambda w: self.filesel.destroy())

        self.filesel.show()

    def callback(self, widget, data=None):
        self.gui.save(self.filesel.get_filename())
        self.filesel.destroy()

class SavePDFWindow(object):
    __slots__ = ["gui", "filesel"]

    def __init__(self, gui):
        self.gui = gui

        self.filesel = gtk.FileSelection(_("Save file as PDF"))
        self.filesel.ok_button.connect("clicked", self.callback)
        self.filesel.cancel_button.connect("clicked",
                                           lambda w: self.filesel.destroy())

        self.filesel.show()

    def callback(self, widget, data=None):
        PDF(self.gui, self.filesel.get_filename())
        self.filesel.destroy()

class SaveImageWindow(object):
    __slots__ = ["gui", "filesel"]

    def __init__(self, gui):
        self.gui = gui

        self.filesel = gtk.FileSelection(_("Save file as an image"))
        self.filesel.ok_button.connect("clicked", self.callback)
        self.filesel.cancel_button.connect("clicked",
                                           lambda w: self.filesel.destroy())

        self.filesel.show()

    def callback(self, widget, data=None):
        Image(self.gui, self.filesel.get_filename())
        self.filesel.destroy()

class SelectValue(object):
    __slots__ = ["gui", "j", "i", "window"]

    def __init__(self, gui, j, i):
        self.gui = gui
        self.j = j
        self.i = i

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(_("Select a number"))
        self.window.connect("delete_event", self.delete_event)
        self.window.show()

        solver = Sudoku(self.gui, difficulty="easy")
        numbers = gtk.Table(self.gui.cellsize[0], self.gui.cellsize[1], True)
        for j in xrange(self.gui.cellsize[1]):
            for i in xrange(self.gui.cellsize[0]):
                number = j * self.gui.cellsize[0] + i + 1

                button = gtk.Button("_" + str(number))
                if number in solver.possible_values(self.j, self.i):
                    button.connect("clicked", self.callback, number)
                else:
                    button.set_sensitive(False)
                button.show()

                numbers.attach(button, i, i + 1, j, j + 1,
                               gtk.EXPAND | gtk.FILL,
                               gtk.EXPAND | gtk.FILL, 1, 1)
        numbers.show()

        button = gtk.Button(stock=gtk.STOCK_CANCEL)
        button.connect("clicked", self.callback)
        button.show()

        vbox = gtk.VBox(False, 0)
        vbox.pack_start(numbers, False, True, 0)
        vbox.pack_start(button, True, False, 0)
        vbox.show()

        self.window.add(vbox)

    def delete_event(self, widget, event, data=None):
        return False

    def callback(self, widget, data=None):
        if data:
            self.gui[self.j, self.i] = data
        self.window.destroy()

class SudokuView(object):
    __slots__ = ["gui", "width", "height", "square_width",
                 "square_height", "view", "area", "pangolayout", "gc"]

    def __init__(self, gui):
        self.gui = gui
        self.width = self.height = 30 * (self.gui.boardsize + 1)
        self.square_width = 30
        self.square_height = 30

        fontname = "Arial 24"
        if self.gui.boardsize > 9:
                fontname = "Arial 12"
                
        self.view = gtk.Alignment(0.5, 0.5)
        self.view.show()

        self.area = gtk.DrawingArea()
        self.area.set_size_request(self.width, self.height)
        self.area.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        self.area.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.pangolayout = self.area.create_pango_layout("")
        self.pangolayout.set_font_description(pango.FontDescription(fontname))
        self.area.connect("expose-event", self.area_expose)
        self.area.connect("button-press-event", self.press)
        self.area.show()

        self.view.add(self.area)

    def draw_board(self):
        # margins
        x = self.square_width / 2                 # 5% left margin
        y = self.square_height / 2                # 5% top margin

        self.area.window.draw_rectangle(self.gc, False, x, y,
                                        self.width - 2 * x,
                                        self.height - 2 * y)

        # horizontal lines
        for i in xrange(self.square_height, self.height - self.square_height,
                        self.square_height):
            if i % (self.square_height * self.gui.cellsize[1]) == 0:
                line_width = 2
            else:
                line_width = 1
            self.gc.set_line_attributes(line_width, self.gc.line_style,
                                        self.gc.cap_style,
                                        self.gc.join_style)
            self.area.window.draw_line(self.gc, x, y + i,
                                       x + self.width - self.square_width,
                                       y + i)

        # vertical lines
        for i in xrange(self.square_width, self.width - self.square_width,
                        self.square_width):
            if i % (self.square_width * self.gui.cellsize[0]) == 0:
                line_width = 2
            else:
                line_width = 1
            self.gc.set_line_attributes(line_width, self.gc.line_style,
                                        self.gc.cap_style,
                                        self.gc.join_style)
            self.area.window.draw_line(self.gc, x + i, y,
                                       x + i,
                                       y + self.height - self.square_height)

    def draw_numbers(self):
        # margins
        x = self.square_width           # 5% margin + half square
        y = self.square_height          # 5% margin + half square

        for j in xrange(self.gui.boardsize):
            for i in xrange(self.gui.boardsize):
                if self.gui[j, i]:
                    if options.getboolean("sudoku", "use_letters"):
                        text = str(Value(self.gui[j, i]))
                    else:
                        text = str(Value(self.gui[j, i]).integer())
                    self.pangolayout.set_text(text)
                    size = self.pangolayout.get_pixel_size()
                    self.area.window.draw_layout(self.gc,
                                                 x + i * self.square_width -
                                                 size[0] / 2,
                                                 y + j * self.square_height -
                                                 size[1] / 2,
                                                 self.pangolayout)


    def area_expose(self, area, event):
        self.gc = self.area.get_style().fg_gc[gtk.STATE_NORMAL]
        self.draw_board()
        self.draw_numbers()

    def press(self, widget, event):
        # margins
        x = self.square_width / 2                 # 5% left margin
        y = self.square_height / 2                # 5% top margin

        if event.x < x or event.x > (self.width - x) or \
           event.y < y or event.y > (self.height - y):
            return
        j = int((event.y - y) / self.square_height)
        i = int((event.x - x) / self.square_width)

        # outside board regions
        if i >= self.gui.boardsize or j >= self.gui.boardsize:
            return
        SelectValue(self.gui, j, i)

    def widget(self):
        return self.view

class Progress(object):
    __slots__ = ["gui", "progress"]

    def __init__(self, gui):
        self.gui = gui

        done = float(self.progress_done()) / self.gui.boardsize ** 2

        self.progress = gtk.ProgressBar()
        self.progress.set_fraction(done)
        #self.progress.pulse()
        self.progress.set_text(str(int(done * 100)) + "%")
        self.progress.show()

    def progress_done(self):
        values = 0
        for j in xrange(self.gui.boardsize):
            for i in xrange(self.gui.boardsize):
                if self.gui[j, i]:
                    values += 1
        return values

    def widget(self):
        return self.progress

class Handicap(object):
    __slots__ = ["gui", "cb", "window", "spin"]

    def __init__(self, gui, callback):
        self.gui = gui
        self.cb = callback

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(_("Create sudoku"))
        self.window.connect("delete_event", self.delete_event)
        self.window.show()

        label = gtk.Label(_("Select your handicap"))
        label.show()

        self.spin = gtk.SpinButton(gtk.Adjustment(options.getint("sudoku",
                                                                 "handicap"),
                                                  0, 30, 1, 5))
        self.spin.show()

        button = gtk.Button(stock=gtk.STOCK_OK)
        button.connect("clicked", self.callback, "hide")
        button.connect("clicked", self.callback)
        button.connect("clicked", self.callback, "destroy")
        button.show()

        vbox = gtk.VBox(False, 10)
        vbox.pack_start(label, False, True, 0)
        vbox.pack_start(self.spin, False, True, 0)
        vbox.pack_start(button, False, True, 0)
        vbox.show()

        self.window.add(vbox)

    def delete_event(self, widget, event, data=None):
        return False

    def callback(self, widget, command=None):
        if command == "destroy":
            self.window.destroy()
        elif command == "hide":
            self.window.hide()
        else:
            self.cb(widget, self.spin.get_value_as_int())

class Create(object):
    __slots__ = ["gui"]

    def __init__(self, gui):
        self.gui = gui
        Handicap(gui, self.callback)

    def callback(self, widget, handicap):
        message = gtk.MessageDialog(self.gui.window, gtk.DIALOG_MODAL,
                                    gtk.MESSAGE_INFO, gtk.BUTTONS_NONE,
                                    _("Creating sudoku..."))
        message.show()

        self.gui.create(handicap)

        message.destroy()

class Check(object):
    __slots__ = ["message"]

    def __init__(self, gui):
        # Use all the algos
        sudoku = Sudoku(gui, difficulty="hard")
        if sudoku.solve():
            self.message = gtk.MessageDialog(gui.window, gtk.DIALOG_MODAL,
                                             gtk.MESSAGE_INFO,
                                             gtk.BUTTONS_CLOSE,
                                             _("This sudoku can be solved."))
            self.message.show()

        else:
            self.message = \
                         gtk.MessageDialog(gui.window, gtk.DIALOG_MODAL,
                                           gtk.MESSAGE_ERROR,
                                           gtk.BUTTONS_CLOSE,
                                           _("This sudoku can't be solved."))
            self.message.show()

        self.message.connect("response", self.callback)

    def callback(self, widget, id=None):
        self.message.destroy()

class Solve(object):
    __slots__ = ["message"]

    def __init__(self, gui):
        if not gui.solve():
            self.message = \
                         gtk.MessageDialog(gui.window, gtk.DIALOG_MODAL,
                                           gtk.MESSAGE_ERROR,
                                           gtk.BUTTONS_CLOSE,
                                           _("This sudoku can't be solved."))
            self.message.connect("response", self.callback)
            self.message.show()

        gui.update()

    def callback(self, widget, id=None):
        self.message.destroy()

class About(object):
    __slots__ = ["message"]

    def __init__(self):
        self.message = gtk.AboutDialog()
        self.message.set_name(Info["name"])
        self.message.set_version(Info["version"])
        self.message.set_copyright(Info["copyright"])
        self.message.set_license(Info["license"])
        self.message.set_website(Info["website"])
        self.message.set_authors(Info["authors"])
        self.message.show()

class WhatIs(object):
    __slots__ = ["message"]

    def __init__(self, gui):
        self.message = gtk.MessageDialog(gui.window, gtk.DIALOG_MODAL,
                                         gtk.MESSAGE_INFO,
                                         gtk.BUTTONS_CLOSE,
                                         Info["whatis"])
        self.message.show()

        self.message.connect("response", self.callback)

    def callback(self, widget, id=None):
        self.message.destroy()

class Solved(object):
    __slots__ = ["message"]

    def __init__(self, gui):
        self.message = gtk.MessageDialog(gui.window, gtk.DIALOG_MODAL,
                                         gtk.MESSAGE_INFO,
                                         gtk.BUTTONS_CLOSE,
                                         _("Solved!"))
        self.message.show()

        self.message.connect("response", self.callback)

    def callback(self, widget, id=None):
        self.message.destroy()

class Gui(History):
    """The GUI."""
    __slots__ = ["window", "menu", "sudoku", "undo_button", "redo_button",
                 "progress", "hbox", "view"]

    def __init__(self, filename=None):
        """Create a new gui.

        Keyword arguments:
        filename -- the file name of the sudoku to open (default None)

        """
        History.__init__(self, filename,
                         cellsize=(options.getint("sudoku", "region_width"),
                                   options.getint("sudoku", "region_height")))

        if not filename:
            self.create(options.getint("sudoku", "handicap"))

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(Info["name"])
        self.window.connect("delete_event", lambda w,e: gtk.main_quit())
        self.window.show()
        self.window.connect("delete_event", self.delete_event)
        self.window.connect("destroy", self.callback, "quit")

        self.menu = Menu(self).widget()

        self.sudoku = SudokuView(self).widget()

        self.undo_button = gtk.Button(stock=gtk.STOCK_UNDO)
        self.undo_button.connect("clicked", self.callback, "undo")
        self.undo_button.set_sensitive(False)
        self.undo_button.show()
        self.redo_button = gtk.Button(stock=gtk.STOCK_REDO)
        self.redo_button.connect("clicked", self.callback, "redo")
        self.redo_button.set_sensitive(False)
        self.redo_button.show()

        self.progress = Progress(self).widget()

        self.hbox = gtk.HBox(False, 10)
        self.hbox.pack_start(self.undo_button, False, False, 10)
        self.hbox.pack_start(self.progress, False, False, 10)
        self.hbox.pack_end(self.redo_button, False, False, 10)
        self.hbox.show()

        self.view = gtk.VBox(False, 0)
        self.view.pack_start(self.menu, False, False, 3)
        self.view.pack_start(self.sudoku, False, False, 3)
        self.view.pack_end(self.hbox, False, False, 3)
        self.view.show()

        self.window.add(self.view)

        gtk.main()

    def __setitem__(self, (row, column), value):
        """Set the number to a position.

        Arguments:
        (row, column) -- a tuple/list/iterable with 2 values
        value -- the number

        """
        History.__setitem__(self, (row, column), value)
        self.update()

    def solve(self):
        """Solve the sudoku."""
        solver = Sudoku(self, difficulty="hard")
        retval = solver.solve()
        self.load_board(solver.to_board())

        return retval

    def create(self, handicap=0):
        """Create a new sudoku with handicap.

        The handicap are the extra numbers given.

        Keyword arguments:
        handicap -- the handicap (default 0)

        """
        creator = Sudoku(Board(cellsize=(options.getint("sudoku",
                                                        "region_width"),
                                         options.getint("sudoku",
                                                        "region_height"))),
                         difficulty=options.get("sudoku",
                                                "difficulty"))
        retval = creator.create(handicap)
        self.load_board(creator.to_board())
        self.forget()

        return retval

    def finished(self):
        """Return if the sudoku if finished."""
        for j in xrange(self.boardsize):
            for i in xrange(self.boardsize):
                if not self[j, i]:
                    return False

        return True

    def update(self, change_history=True):
        """Update the board.

        The actual values of the sudoku will be added to the history.
        Sometimes this is not wanted (sudoku created or loaded), to do this,
        call update(False).

        Keyword arguments:
        change_history -- if True the change will be added to history
                          (default True)

        """
        if change_history:
            self.remember()

        self.sudoku.destroy()
        self.progress.destroy()

        self.sudoku = SudokuView(self).widget()

        self.undo_button.set_sensitive(not self.is_first_position())
        self.redo_button.set_sensitive(not self.is_last_position())
        self.progress = Progress(self).widget()

        self.view.pack_start(self.sudoku, False, False, 3)

        self.hbox.pack_start(self.progress, False, False, 10)

        if self.finished():
            Solved(self)

    def callback(self, widget, function):
        """Gtk callback function.

        Arguments:
        widget -- the widget
        function -- string with the command to do ("open", "save", etc)

        """
        if function == "open":
            OpenWindow(self)
        elif function == "save":
            SaveWindow(self)
        elif function == "save_pdf":
            SavePDFWindow(self)
        elif function == "save_image":
            SaveImageWindow(self)
        elif function == "print":
            Printer(self.to_board())
        elif function == "quit":
            gtk.main_quit()
        elif function == "undo":
            if self.undo():
                self.update(False)
        elif function == "redo":
            if self.redo():
                self.update(False)
        elif function == "create":
            Create(self)
        elif function == "check":
            Check(self)
        elif function == "solve":
            Solve(self)
        elif function == "give_one":
            # Use all the algols
            sdk1 = Sudoku(self, difficulty="hard")
            sdk1.solve()
            sdk2 = Sudoku(self, difficulty="easy")
            sdk2.give_numbers(sdk1.to_board(), 1)
            self.load_board(sdk2.to_board())
            self.update()
        elif function == "about":
            About()
        elif function == "whatis":
            WhatIs(self)

    def delete_event(self, widget, event, data=None):
        return False
